package me.icymint.libra.sas.config;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import me.icymint.libra.sas.exception.SasConfigException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * SAS扫描器配置文件，用于配置具体的SAS执行临时文件夹配置。
 * 
 * @author Daniel
 * @datetime 2012-10-22下午2:07:18
 * 
 */
public abstract class SasConfig extends Properties {
	/**
	 * 
	 */
	private static final long serialVersionUID = -4036378860219124879L;
	protected final Logger logger = LoggerFactory.getLogger(getClass());
	/**
	 * 文件分隔符
	 */
	public final static char G_ = File.separatorChar;

	/**
	 * 行分隔符
	 */
	public final static String GL = System.getProperty("line.separator");
	/**
	 * 运行标记，目录RUN_FLAG
	 */
	public final static String STOP = "STOP.FLAG";

	public final static String START = "START.FLAG";
	public final static String ERROR = "ERROR.FLAG";
	public final static String FINISH = "FINISH.FLAG";
	/**
	 * 根文件。
	 */
	public final static String SAS = "BIN.SAS";
	/**
	 * 日志目录。
	 */
	public final static String LOG = "MAIN_LOG.TXT";

	public final static String LOG_CHECK = "MAIN_LOG_CHECK.TXT";

	/**
	 * 生成SDE环境规范路径字符串
	 * 
	 * @param abc
	 * @return SDE的规范路径
	 */
	public static String getSDEPath(File abc) {
		String path = abc.getPath().replace("~", "\\~");
		if (abc.isDirectory() && !path.matches(".*\\" + G_))
			path = path + G_;
		return path;
	}

	private final File cache;
	public static SasSystemConfig SAS_SYSTEM_CONFIG;
	static {
		try {
			SAS_SYSTEM_CONFIG = new SasSystemConfig();
		} catch (IOException e) {
			System.out.println("SAS系统配置出错！");
			e.printStackTrace();
		}
	}

	private SasSystemConfig ssc;

	private final File flag;
	private final File flagfile;

	private final File start;

	private final File stop;

	private final File error;

	private final File finish;

	private final File logdir;

	private final File sas;

	private final File mainlog;

	private static Pattern pattern = Pattern.compile("[$]\\{([_a-zA-Z]+?)\\}");

	public SasConfig(File cache) {
		this(cache, null);
	}

	public SasConfig(File cache, SasSystemConfig ssc) {
		this.cache = cache;
		this.flag = new File(cache, "运行标志");
		this.flagfile = new File(flag, ".FLAG");
		this.start = new File(flag, START);
		this.stop = new File(flag, STOP);
		this.error = new File(flag, ERROR);
		this.finish = new File(flag, FINISH);
		this.logdir = new File(cache, "日志");
		this.mainlog = new File(logdir, LOG);
		this.sas = new File(cache, SAS);
		if (ssc == null)
			ssc = SAS_SYSTEM_CONFIG;
		this.ssc = ssc;
		this.put("G_MLIB_DIR", cache.getAbsolutePath());
	}

	public void createFlag() {
		try {
			this.flagfile.createNewFile();
		} catch (IOException e) {
		}
		;
	}

	/**
	 * 设置临时文件夹根目录。
	 */
	public File getCacheRoot() {
		return cache;
	}

	public File getErrorFlag() {
		return error;
	}

	public File getFinishFile() {
		return finish;
	}

	public File getFlagRoot() {
		return flag;
	}

	public File getLogRoot() {
		return logdir;
	}

	public File getMainLog() {
		return mainlog;
	}

	/**
	 * 获取键值key的属性值。 首先获取自己的配置值，如果不存在，则获取SystemConfig中的默认值。
	 */
	@Override
	public String getProperty(String key) {
		String abc = super.getProperty(key);
		if (abc == null || abc.equals("")) {
			abc = ssc.getProperty(key);
			if (abc != null && abc.equals(""))
				abc = null;
		}
		return abc;
	}

	public File getSasFile() {
		return sas;
	}

	protected File getStartFile() {
		return start;
	}

	protected File getStopFile() {
		return stop;
	}

	public SasSystemConfig getSystemConfig() {
		return ssc;
	}

	public boolean isError() {
		return error.exists();
	}

	public boolean isFinished() {
		return finish.exists();
	}

	public boolean isFlagExist() {
		return this.flagfile.exists();
	}

	public boolean isStarted() {
		return start.exists();
	}

	public abstract void output(String sas);

	/**
	 * 自定义模板流，生成SAS文件。
	 * 
	 * @param bin
	 * @param sas
	 * @param is
	 */
	protected void output(String sas, InputStream is, OutputStream os) {
		if (is == null)
			is = SasConfig.class.getResourceAsStream("bin.sas");
		BufferedReader in = new BufferedReader(new InputStreamReader(is));
		String line = null;
		try {
			BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(os,
					"GBK"));
			while ((line = in.readLine()) != null) {
				Matcher m = pattern.matcher(line);
				String res = new String(line);
				while (m.find()) {
					String abc = this.getProperty(m.group(1));
					if (abc == null) {
						if (m.group(1).equals("SDE_CONTENT"))
							abc = sas;
						else
							throw new SasConfigException();
					}
					res = res.replace(m.group(),
							abc.replace("/", File.separator));
				}
				bw.append(res);
				bw.newLine();
			}
			bw.close();
			in.close();
		} catch (UnsupportedEncodingException e) {
		} catch (IOException e) {
			e.printStackTrace();
		} catch (SasConfigException e) {
			logger.error("SAS配置不完全!\n" + line + "行无法识别");
			e.printStackTrace();
		}
	}

	public void shutdown() {
		if (!stop.exists())
			try {
				stop.createNewFile();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
	}
}
